## $Id: test_permissions.py 11954 2014-11-13 16:54:17Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
Permission tests.

Check accessibility of views, pages, viewlets.

:test-layer: python

"""
import shutil
import tempfile
from zope.app.testing.functional import HTTPCaller as http
from zope.component import createObject
from zope.component.hooks import setSite, clearSite
from zope.security.interfaces import Unauthorized
from zope.testbrowser.testing import Browser
from waeup.ikoba.app import Company
from waeup.ikoba.testing import (
    FunctionalLayer, FunctionalTestCase, get_all_loggers, remove_new_loggers,
    remove_logger)

manager_pages = [
    # The pages that should only be accessible by manager...
    '/@@manage', '/@@administration',
    '/users/@@index', '/users/@@add', '/users/alice/@@manage',
    '/datacenter/@@index', '/datacenter/@@upload', '/datacenter/@@import1',
    '/datacenter/@@import2', '/datacenter/@@import3', '/datacenter/@@import4',
    '/datacenter/@@logs', '/datacenter/@@show', '/datacenter/@@manage',
    ]
public_pages = [
    # Pages accessible also by the public...
    '/@@index', '/@@login', '/@@logout',
    ]

class PermissionTest(FunctionalTestCase):
    """Here we try to request all pages and check, whether they are
    accessible.
    """

    layer = FunctionalLayer

    def setUp(self):
        super(PermissionTest, self).setUp()
        # Set up a complete company to have every page available...
        app = Company()
        self.getRootFolder()['app'] = app
        setSite(self.getRootFolder()['app'])
        self.dc_root = tempfile.mkdtemp()
        app['datacenter'].setStoragePath(self.dc_root)
        app['users'].addUser('alice', 'alice')

        self.browser = Browser()
        self.browser.handleErrors = False
        pass

    def tearDown(self):
        super(PermissionTest, self).tearDown()
        clearSite()
        shutil.rmtree(self.dc_root)
        return

    def isAccessible(self, path):
        path = 'http://localhost/app%s' % path
        try:
            self.browser.open(path)
            return True
        except Unauthorized:
            return False
        return

    def testUnauthenticatedUser(self):
        for path in manager_pages:
            if not self.isAccessible(path):
                continue
            self.fail('Path %s can be accessed by anonymous.' % path)
        for path in public_pages:
            if self.isAccessible(path):
                continue
            self.fail('Path %s cannot be accessed by anonymous.' % path)
        return
