## $Id: product.py 12592 2015-02-11 11:59:34Z henrik $
##
## Copyright (C) 2014 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
These are the products (e.g. licenses).
"""
import grok
from time import time
from grok import index
from datetime import date
from hurry.workflow.interfaces import IWorkflowInfo, IWorkflowState
from zope.event import notify
from zope.catalog.interfaces import ICatalog
from zope.component import getUtility
from zope.component.interfaces import IFactory, ComponentLookupError
from zope.interface import implementedBy
from zope.i18n import translate
from waeup.ikoba.interfaces import IIkobaUtils
from waeup.ikoba.interfaces import MessageFactory as _
from waeup.ikoba.utils.helpers import attrs_to_fields, get_current_principal
from waeup.ikoba.products.interfaces import (
    IProduct, IProduct,)

class Product(grok.Container):
    """This is a product.
    """
    grok.implements(IProduct)
    grok.provides(IProduct)
    #grok.baseclass()

    form_fields_interface = IProduct

    local_roles = [
        'waeup.local.ProductManager',
        ]

    def __init__(self, *args, **kw):
        super(Product, self).__init__(*args, **kw)
        self.tc_dict = {}

    @property
    def contract_category_title(self):
        cc_dict = getUtility(IIkobaUtils).CON_CATS_DICT
        return cc_dict[self.contract_category]

    @property
    def contract_autotitle(self):
        if self.contract_title:
            return self.contract_title
        return self.title

    @property
    def active(self):
        now = date.today()
        return (not self.valid_from or self.valid_from <= now) and \
               (not self.valid_to or self.valid_to >= now)

Product = attrs_to_fields(Product)


class ProductFactory(grok.GlobalUtility):
    """A factory for products.
    """
    grok.implements(IFactory)
    grok.name(u'waeup.Product')
    title = u"Create a new product.",
    description = u"This factory instantiates new products."

    def __call__(self, *args, **kw):
        return Product(*args, **kw)

    def getInterfaces(self):
        return implementedBy(Product)

@grok.subscribe(IProduct, grok.IObjectRemovedEvent)
def handle_product_removed(product, event):
    """If a product is deleted, we make sure that also referrers to
    customer contract objects are removed.
    """
    prodid = product.product_id

    # Find all customer contracts that refer to given product...
    try:
        cat = getUtility(ICatalog, name='contracts_catalog')
    except ComponentLookupError:
        # catalog not available. This might happen during tests.
        return
    results = cat.searchResults(last_product_id=(prodid, prodid))
    for contract in results:
        # Remove that referrer...
        contract.product_object = None
        notify(grok.ObjectModifiedEvent(contract))
        contract.customer.__parent__.logger.info(
            'ObjectRemovedEvent - %s - %s - removed: %s' % (
                contract.customer.customer_id,
                contract.contract_id,
                prodid))
    return

